/*  
    A class to represent a past version of the player on the screen in the final level; looks like the player
    @Author Jonathan Saunders and Ethan White
    @class
*/
function FinalCopy()
{    
    this.hasItem = false;   // True if the copy is holding the item, false otherwise
    
    this.touching = false;   //True if the item was just lost, and can't be regained during this collision with it
    
    this.animation = 4;     // Keeps track of which animation is currently being played
    /*
     * Animations reference:
     * 1 = Idle left
     * 2 = Idle right
     * 3 = Idle up
     * 4 = Idle down
     * 5 = Run left
     * 6 = Run right
     * 7 = Run up
     * 8 = Run down
     * More can be added as necessary
     */
    
    this.frames = 8;        //Number of frames in player animations
    
    this.dead = false;		// True if the copy should pop
    
    this.touching = false;     //True if the item was just lost, and can't be regained during this collision
    
    this.level = null;		// A reference to the level object  @type Level
    
    this.ticks = 0;			// What tick the game is currently at
    
    // Initialises this object
    this.startupFinalCopy = function(level)
    {
		this.startupAnimatedGameObject(g_ResourceManager.idleDown, 250, 50, 5, this.frames, 10);
        this.level = level;
		this.ticks = 0;
		
		
		//Adjust the collision area
        this.collideY = 35 * g_RatioY;
        this.collideHeight = -40 * g_RatioY;
        this.collideX = 5 * g_RatioX;
        this.collideWidth = -10 * g_RatioX;
		this.dead = true;
        return this;
    }

    //Called when this copy reaches the level end post
    this.hitLevelEnd = function()
    {
    		if (this.hasItem)
    		{
    			g_Gem.shutdown();
    			g_player.canGetItem = false;
    			
    			//Plays fail sound (start and end positions, can be found in SoundManager)
    			g_SoundManager.playSound(3.580, 4.615);
    			
    			g_player.rolling = true;
    		}
    		this.shutdown();
    		g_Copies.pop();

    }


    /*  Updates the current animation depending on the movement
        of the player. */
    this.updateAnimation = function(index)
    {
    	
    	switch(index) {
    		
    		//Idle left
    		case 1:
    			if (this.animation != 1)
    			{
	    			this.setAnimation(g_ResourceManager.idleLeft, this.frames, 10);
	    			this.animation = 1;
    			}
    			break;
    			
    		//Idle right
    		case 2:
    			if (this.animation != 2)
    			{
    				this.setAnimation(g_ResourceManager.idleRight, this.frames, 10);
    				this.animation = 2;
    			}
    			break;
    			
    		//Idle up
    		case 3:
    			if (this.animation != 3)
    			{
    				this.setAnimation(g_ResourceManager.idleUp, this.frames, 10);
    				this.animation = 3;
    			}
    			break;
    			
    		//Idle down
    		case 4:
    			if (this.animation != 4)
    			{
    				this.setAnimation(g_ResourceManager.idleDown, this.frames, 10);
    				this.animation = 4;
    			}
    			break;
    			
    		//Move left
    		case 5:
    			if (this.animation != 5)
    			{
    				this.setAnimation(g_ResourceManager.moveLeft, this.frames, 10);
    				this.animation = 5;
    			}
    			break;
    			
    		//Move right
    		case 6:
    			if (this.animation != 6)
    			{
    				this.setAnimation(g_ResourceManager.moveRight, this.frames, 10);
    				this.animation = 6;
    			}
    			break;
    			
    		//Move up
    		case 7:
    			if (this.animation != 7)
    			{
    				this.setAnimation(g_ResourceManager.moveUp, this.frames, 10);
    				this.animation = 7;
    			}
    			break;
    			
    		//Move down
    		case 8:
    			if (this.animation != 8)
    			{    				
    				this.setAnimation(g_ResourceManager.moveDown, this.frames, 10);
    				this.animation = 8;
    			}
    			break;
    			
    		//Invalid animation setting
    		default:
    			break;
    	}
    }


    /*  Updates the object
        @param dt The time since the last frame in seconds
        @param context The drawing context
        @param xScroll The global scrolling value of the x axis
        @param yScroll The global scrolling value of the y axis  */
    this.update = function (/**Number*/ dt, /**CanvasRenderingContext2D*/context, /**Number*/ xScroll, /**Number*/ yScroll)
    {
    	
    	if (this.ticks > 1.5)
    	{
    		this.hitLevelEnd();
    	}
    	else if (this.ticks > 1)
    	{
    		this.updateAnimation(5);
    		this.x -= g_Speed * dt;
    	}
    	else if (this.ticks > 0.75)
    	{
    		this.updateAnimation(4);
        	this.dead = false;
    	}
    	if (this.hasItem)
    	{
    		g_Gem.x = this.x+8;
    		g_Gem.y = this.y+10;
    	}
       	this.ticks += dt;
       	this.collisionCheck();
    }
    
    //Collision check for Timetanium
    this.collisionCheck = function()
    {
    	
    	//Check collisions with timetanium
    	if (g_Gem.owner == null)
    	{
    		if (this.collisionArea().intersects(g_Gem.collisionArea()))
    		{
    			g_Gem.changeOwner(this);
    		}
    	}
    	//Check collisions with timetanium carrier
    	else if (g_Gem.owner != this)
    	{
    		if (this.collisionArea().intersects(g_Gem.owner.collisionArea()))
    		{
    			if (g_Gem.cooldown <= 0) {
	    			if (!this.touching)
	    			{
	    				g_Gem.changeOwner(this);
	    				g_Gem.cooldown = g_Gem.maxCooldown;
	    				this.touching = true;
	    			}
    			}
    		}
    		else this.touching = false;
    	} 
    	
    }
    
    this.shutdown = function()
    {
        this.shutdownAnimatedGameObject();
    }
    
    //Called when this entity picks up an item
    this.getItem = function()
    {
    	this.hasItem = true;
    	
    	//play the copy get item sound (start and end positions, can be found in SoundManager)
    	g_SoundManager.playSound(1.135, 1.640);
    }
    
    //Called when this entity loses its held item
    this.loseItem = function()
    {
    	this.hasItem = false;
    	this.touching = true;
    }
    
}

FinalCopy.prototype = new AnimatedGameObject;